/*

  File Integrals.ccp:

  Computes and integral through the Trapezoidal rule or the Cavalieri-Simpson rule.

*/


#include "Integrals.h"


/*
  Function Trapezoidal
  computes the integral of a function known on N+1 points
  through the trapezoidal rule.
*/

double Trapezoidal( double f[], double h, long N )
{
  double sum;

  sum = 0.5 * ( f[0] + f[N] );
  for ( long i = 1; i < N; i++ ) sum += f[i];
  return( h * sum );
}


/*
  Function Simpson
  computes the integral of a function known on N+1 points
  (N even) through the Cavalieri-Simpson's rule.
*/


double Simpson( double f[], double h, long N )
{
  double sump = 0.0, sumd = 0.0;

  for ( long i = 2; i <= N - 2; i = i + 2 ) sump += f[i];
  for ( long i = 1; i <= N - 1; i = i + 2 ) sumd += f[i];
  return( ( h / 3.0 ) * ( f[0] + f[N] + 2.0 * sump + 4.0 * sumd ));
}
