/*

  File simpson.ccp:

  Computes the integral of a function through the Cavalieri-Simpson's rule

*/


#include <iostream>
#include <cmath>
#include "Integrals.h"

using namespace std;


// Functions to integrate

#define F( x ) exp( x )                        // Function
#define a     0.0                              // Left edge
#define b     2.0                              // Right edge
#define Fint( a, b ) ( exp( b ) - exp( a ) )   // Integral


int main()
{
  cout << "Computes the definite integral of a function through the Cavalieri-Simpson's rule" << endl;

  long N;
  cout << "Number of intervals (even!): ";
  cin >> N;

  try {
     if ( ( N % 2 ) != 0 ) throw "Error: the number of intervals MUST be even!!!";
     }
  catch ( char const * str ) {
     cout << str << endl;
     return 0;
     }


  // Defines the function on the gridpoints

  double x;
  double *f = new double( N + 1 );
  double dx = ( b - a ) / N;

  for ( long i = 0; i <= N; i++ )
    {
      x    = a + i * dx;
      f[i] = F( x );
    }


  // Computes the definite integral

  double Inum = Simpson( f, dx, N );
  double Iex  = Fint( a, b );


  // Prints the result

  cout.precision(15);
  cout << "Numerical value of the integral: I_num = " << Inum << endl;
  cout << "Exact value of the integral: I_ex = " << Iex << endl;
  cout << "Error: abs( I_ex  - I_num ) = " << abs( Iex - Inum ) << endl;

  return 0;
}
