/*

  File Trapezoidal.ccp:

  Computes the integral of a given function through the trapezoidal rule.

*/


#include <iostream>
using namespace std;

#include <cmath>
using namespace std;

#include "Integrals.h"


// Functions to integrate

#define F( x ) exp( x )                        // Function
#define a     0.0                              // Left extrema
#define b     2.0                              // Right extrema
#define Fint( a, b ) ( exp( b ) - exp( a ) )   // Exact integral


int main()
{
  cout << "Computes the definite integral of a function through the trapezoidal rule" << endl;

  long N;
  cout << "Number of intervals: ";
  cin >> N;


  // Defines the function on the points

  double x;
  double *f = new double[ N + 1 ];
  double dx = ( b - a ) / N;

  for ( long i = 0; i <= N; i++ )
    {
      x    = a + i * dx;
      f[i] = F( x );
    }


  // Compute the definite integral 

  double Inum = Trapezoidal( f, dx, N );
  double Iex  = Fint( a, b );


  // Prints the result

  cout.precision(15);
  cout << "Numerical value of the integral: I_num = " << Inum << endl;
  cout << "Exact value of the integral: I_ex = " << Iex << endl;
  cout << "Error: abs( I_ex  - I_num ) = " << abs( Iex - Inum ) << endl;

  return 0;
}
